/*  
 *  File:       SxNano_8in_V33.ino
 *  Autor:      Frank Keil
 *  Controller: Arduino Nano (5V, 16MHz), ATmega328
 *  Hardware:   Sx-Decoder von Frank Keil 
 *              http://frank-keil.de/Selectrix/Sx-Decoder/Sx-Decoder.html
 *  
 *  Libraries:  SXArduino: Quellen von G.v.d.Sel  
 *              https://github.com/gvandersel/SXArduino
 *
 *  Created on: 02.01.2012
 *  Last changed on: 09.03.2022
 *  
 *  Beschreibung:
 *  Liest 8 Eingänge ein und schreibt es in eine Sx-Adresse
 *
 *
 *  Pin2=T0=INT0, Pin4=T1, Pin3=SX-Write-High, Pin6=SX-Write-Low
 *
 *  Zum Programmieren des Moduls am Sx-Bus können die SX-Kanäle 0 bis 5 verwendet werden:
 *   Sx-Kanal 0 : Sx-Adresse 1 dieses Moduls (0..99)
 *   Sx-Kanal 1 : -
 *   Sx-Kanal 2 : -
 *   Sx-Kanal 3 : -
 *   Sx-Kanal 4 : -
 *   Sx-Kanal 5 : -
 *  Die alten Daten auf dem Bus werden gerettet und zurück geschrieben.
 *   
 *  Bit_15 signalisiert die Gleisspannung
 *  Bit_16 ist während der Programmierung an
 *
 *  SXbus.write(adr, val);      to set a value on an address of the SX-bus.
 *  val = SXbus.read(adr);      to get a value from the SXbus.
 *  SXbus.writePWR(val);        val = 0 for off, val = 1 for on (all other values have no influence)
 *  val = SXbus.readPWR();      val = 0 for off, val = 1 for on.
 *  
 */

  #include "SXArduino.h"                      // Lib einbinden
  SXArduino SXbus(2,4,6,3);                   // Pinbelegung FK (Takt, Daten, Schreiben, Schreiben)
  // SXArduino SXbus(2,4,3,6);                // Pinbelegung openSX(Takt, Daten, Schreiben, Schreiben)  bool noCC2000 = false;                      // false 0 ... 103, true 0 ... 111

  #include <EEPROM.h>                         // EEPROM Bibliothek hinzufügen

  int analogPin = 6;                          // Progtaster
  #define Bit_1 1                             // Led 1 (linke Buchsen-Leiste Pin 3 
  #define Bit_2 0                             // Led 2 (linke Buchsen-Leiste Pin 4
  #define Bit_3 5                             // Led 3 (linke Buchsen-Leiste Pin 10
  #define Bit_4 7                             // Led 4 (linke Buchsen-Leiste Pin 12
  #define Bit_5 8                             // Led 5 (linke Buchsen-Leiste Pin 13
  #define Bit_6 9                             // Led 6 (linke Buchsen-Leiste Pin 14
  #define Bit_7 10                            // Led 7 (linke Buchsen-Leiste Pin 15
  #define Bit_8 11                            // Led 8 (linke Buchsen-Leiste Pin 16
  #define Bit_15 12                           // Bit_15 Led - Gleis ein
  #define Bit_16 13                           // Bit_16 Led - Prog aktiv

  byte Sensor1;                               // Zustand Sensor1
  byte Sensor2;                               // Zustand Sensor2
  byte Sensor3;                               // Zustand Sensor3
  byte Sensor4;                               // Zustand Sensor4
  byte Sensor5;                               // Zustand Sensor5
  byte Sensor6;                               // Zustand Sensor6
  byte Sensor7;                               // Zustand Sensor7
  byte Sensor8;                               // Zustand Sensor8

  byte DecAdr=10;                             // Decoder Adresse
  byte SxAdr1=10;                             // Sx Adresse 1
  byte SxData1=0;                             // aktuelle SxDaten 1                  
  byte SxAdrSchreib=0;                        // SxAdressen die geschrieben werden 
  byte SxDataSchreib=0;                       // SxDaten die geschrieben werden  
  byte tb;                                    // Gleisbit
  byte oldtb;                                 // alter Wert Gleisbit
  int  progAkt;                               // Programmierung aktiv
  int  Prog = 0;                              // Analogwert Progtaster
  byte Vergleich=0;

void sxisr(void) 
    { 
    SXbus.isr();
    } 

void setup() 
    {
    // Serial.begin(115200);                  // Baudrate
    // Serial.println("SxNano_8in_V33");      // Ausgabe
    pinMode(Bit_1,INPUT_PULLUP);              // D0 als Eingang setzen
    pinMode(Bit_2,INPUT_PULLUP);              // D1 als Eingang setzen
    pinMode(Bit_3,INPUT_PULLUP);              // D5 als Eingang setzen
    pinMode(Bit_4,INPUT_PULLUP);              // D7 als Eingang setzen
    pinMode(Bit_5,INPUT_PULLUP);              // D8 als Eingang setzen
    pinMode(Bit_6,INPUT_PULLUP);              // D9 als Eingang setzen
    pinMode(Bit_7,INPUT_PULLUP);              // D10 als Eingang setzen
    pinMode(Bit_8,INPUT_PULLUP);              // D11 als Eingang setzen
    pinMode(Bit_15,OUTPUT);                   // D12 als Ausgang setzen
    pinMode(Bit_16,OUTPUT);                   // D13 als Ausgang setzen
    DecAdr = EEPROM.read(1);                  // EEprom Zelle 1 auslesen
    SxAdrSchreib = DecAdr;                    // EEprom Zelle 1 auslesen
    delay(1000);                              // warte 1sec
    SXbus.init();                             // SX-bus Kommunikation
    attachInterrupt(0, sxisr, RISING);        // wechsel INT0 triggers the interrupt routine sxisr (see above)
    }

void loop() 
    {
    tb = SXbus.read(109);                           // Info, ob das Gleissignal an/aus ist
    tb = bitRead(tb,7);
    if (oldtb != tb)                                // Gleisbit geändert?
        {
        oldtb = tb;                                 // Gleisbit gleich setzen
        digitalWrite(Bit_15,tb);                    // Led je nach Zustand ein/ausschalten
        // Serial.println(oldtb);                   // Ausgabe
        }
    
    // SxData1=SXbus.read(DecAdr);                  // Decoder-Adresse auslesen
    Sensor1 = digitalRead(Bit_1);                   // Eingang1 einlesen

        if (Sensor1 == 0)                           // wenn Sensor1 frei ist
            {
        bitClear(SxData1,0);                      // Bit löschen
            }
        else                                        // ansonsten
            {
        bitSet(SxData1,0);                        // Bit setzen
            }
    
    Sensor2 = digitalRead(Bit_2);                   // Eingang2 einlesen
        if (Sensor2 == 0)                           // wenn Sensor2 frei ist
            {
        bitClear(SxData1,1);                      // Bit löschen
            }
        else                                        // ansonsten
            {
        bitSet(SxData1,1);                        // Bit setzen
            }
        
    Sensor3 = digitalRead(Bit_3);                 // Eingang3 einlesen
    if (Sensor3 == 0)                             // wenn Sensor3 frei ist
        {
        bitClear(SxData1,2);                      // Bit löschen
        }
    else                                          // ansonsten
        {
        bitSet(SxData1,2);                        // Bit setzen
        }
    
    Sensor4 = digitalRead(Bit_4);                 // Eingang4 einlesen
    if (Sensor4 == 0)                             // wenn Sensor4 frei ist
        {
        bitClear(SxData1,3);                      // Bit löschen
        }
    else                                          // ansonsten
        {
        bitSet(SxData1,3);                        // Bit setzen
        }
        
    Sensor5 = digitalRead(Bit_5);                 // Eingang5 einlesen
    if (Sensor5 == 0)                             // wenn Sensor5 frei ist
        {
        bitClear(SxData1,4);                      // Bit löschen
        }
    else                                          // ansonsten
        {
        bitSet(SxData1,4);                        // Bit setzen
        }
        
    Sensor6 = digitalRead(Bit_6);                 // Eingang6 einlesen  
    if (Sensor6 == 0)                             // wenn Sensor6 frei ist
            {
        bitClear(SxData1,5);                      // Bit löschen
            }
        else                                      // ansonsten
            {
        bitSet(SxData1,5);                        // Bit setzen
            }
        
    Sensor7 = digitalRead(Bit_7);                 // Eingang7 einlesen   
    if (Sensor7 == 0)                             // wenn Sensor7 frei ist
        {
        bitClear(SxData1,6);                      // Bit löschen
        }
    else                                          // ansonsten
        {
        bitSet(SxData1,6);                        // Bit setzen
        }
        
    Sensor8 = digitalRead(Bit_8);                 // Eingang8 einlesen   
    if (Sensor8 == 0)                             // wenn Sensor8 frei ist
        {
        bitClear(SxData1,7);                      // Bit löschen
        }
    else                                          // ansonsten
        {
        bitSet(SxData1,7);                        // Bit setzen
        }
    // Serial.println(SxData1);                      // Ausgabe
    // Serial.println(" ");                          // Ausgabe       
    SxDataSchreib = SxData1;
    Schreiben();                                  // ins Unterprog springen           
       
    // Programmierroutine
     if (progAkt == 0)                            // wenn progAktiv nicht gesetzt
        {
        tb = SXbus.readPWR();                     // Info, ob das Gleissignal an/aus ist
        Prog = analogRead(analogPin);             // read the input pin 
        if (Prog <= 500 && tb ==false)            // falls gedrückt und Gleis aus
            {
            SxData1 = SXbus.read(106);            // Sx-Adr 106 auslesen
            Vergleich = SxData1 & B00100000;      // Prog schon von anderem Decoder gesetzt?
            if (Vergleich != 32)
                {
                progAkt = 1;                      // setze progAktiv 
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                SxDataSchreib = SxData1 | B00100000;  // Prog-Bit setzen
                SxAdrSchreib = 106;               // Sx Adresse die beschrieben werden soll
                Schreiben();                      // ins Unterprog springen
                delay (500);                      // warte
                SxData1=SXbus.read(0);            // Sx-Adr 00 auslesen
                EEPROM.write(10,SxData1);         // und in eEprom retten                            
                DecAdr = EEPROM.read(1);          // EEprom Zelle 1 auslesen                
                SxAdrSchreib=0;                   // Sx Adresse die beschrieben werden soll
                SxDataSchreib=DecAdr;             // setze Wert der geschrieben werden soll
                Schreiben();                      // ins Unterprog springen
                }            
            else
                {
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,LOW);         // Led ausschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,LOW);         // Led ausschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,HIGH);        // Led einschalten
                delay (800);                      // warte
                digitalWrite(Bit_16,LOW);         // Led ausschalten                
                }
            delay (1500);                         // warte
            // Serial.println(progAkt); 
            }
        } 
    
    if (progAkt == 1)                             // wenn progAktiv gesetzt
        {
        tb = SXbus.readPWR();                     // Info, ob das Gleissignal an/aus ist              
        Prog = analogRead(analogPin);             // read the input pin 
        if (Prog <= 500 || tb ==true)             // falls gedrückt oder Gleis ein
            {
            DecAdr=SXbus.read(0);                 // Sx-Adr 00 auslesen
            EEPROM.write(1,DecAdr);               // in eEprom schreiben
            SxDataSchreib = EEPROM.read(10);      // EEprom Zelle 10 mit geretteten auslesen
            SxAdrSchreib=0;                       // Sx Adresse die beschrieben werden soll
            Schreiben();                          // ins Unterprog springen           
            SxData1=SXbus.read(106);              // Sx-Adr 106 auslesen
            SxDataSchreib=SxData1 & B11011111;    // Prog-Bit zurück setzen
            SxAdrSchreib=106;                     // Sx Adresse die beschrieben werden soll
            Schreiben();                          // ins Unterprog springen
            delay (1000);                         // warte
            progAkt = 0;                          // lösche progAktiv
            digitalWrite(Bit_16,LOW);             // Led ausschalten
            }
        SxAdrSchreib = DecAdr;                    // 
        }
    }

void Schreiben()
    {    
    SXbus.write(SxAdrSchreib,SxDataSchreib);      // Daten auf Sx-Bus in Adr schreiben
    } 
